package com.ideabus.sdk_test;

import static com.ideabus.sdk_test.Global.wbpProtocol;


import android.annotation.SuppressLint;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import com.ideabus.model.bluetooth.MyBluetoothLE;
import com.ideabus.model.data.DRecord;
import com.ideabus.model.data.DeviceInfo;
import com.ideabus.model.data.DiagnosticDRecord;
import com.ideabus.model.data.EachMData;
import com.ideabus.model.data.NocturnalModeDRecord;
import com.ideabus.model.data.User;
import com.ideabus.model.data.VersionData;
import com.ideabus.model.protocol.WBPProtocol;
import com.ideabus.sdk_test.numberpicker.NumerPickerDialog;

import java.util.Calendar;
import java.util.List;
import java.util.Random;


@SuppressLint("NonConstantResourceId")
public class WBPTestActivity extends AppCompatActivity implements
        WBPProtocol.OnConnectStateListener, View.OnClickListener,
        WBPProtocol.OnDataResponseListener, WBPProtocol.OnNotifyStateListener,
        MyBluetoothLE.OnWriteStateListener {

    private String TAG = this.getClass().getSimpleName();

    private ListView bpmList;
    private LogListAdapter logListAdapter;
    NumerPickerDialog npDialog;

    private boolean isSendPersonParam;
    private Toolbar toolbar;
    private boolean isConnecting;
    private static int protocolVersion =0;
    private final int[] restIntervals = new int[]{0, 15, 30, 60, 120, 180, 240, 300};


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        //Initialize the body ester machine Bluetooth module

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_wbptest);

        initView();
        initParam();
        initListener();
    }

    private void initView() {
        toolbar = (Toolbar) findViewById(R.id.tool_bar);
        bpmList = (ListView) findViewById(R.id.bpm_list_view);
        findViewById(R.id.buttonView).setVisibility(View.GONE);
        findViewById(R.id.button19).setVisibility(View.GONE);
    }

    private void initParam() {
        setSupportActionBar(toolbar);

        //Initialize the connection SDK
        wbpProtocol = WBPProtocol.getInstance(this, false, true, Global.sdkid_WBP);
        toolbar.setSubtitle("Watch Blood Pressure "+ wbpProtocol.getSDKVersion());

        logListAdapter = new LogListAdapter(this);
        bpmList.setAdapter(logListAdapter);
    }

    private void initListener() {
        findViewById(R.id.button1).setOnClickListener(this);
        findViewById(R.id.button2).setOnClickListener(this);
        findViewById(R.id.button3).setOnClickListener(this);
        findViewById(R.id.button4).setOnClickListener(this);
        findViewById(R.id.button5).setOnClickListener(this);
        findViewById(R.id.button6).setOnClickListener(this);
        findViewById(R.id.button7).setOnClickListener(this);
        findViewById(R.id.button8).setOnClickListener(this);
        findViewById(R.id.button9).setOnClickListener(this);
        findViewById(R.id.button11).setOnClickListener(this);
        findViewById(R.id.button12).setOnClickListener(this);
        findViewById(R.id.button13).setOnClickListener(this);
        findViewById(R.id.button14).setOnClickListener(this);
        findViewById(R.id.button15).setOnClickListener(this);
        findViewById(R.id.button16).setOnClickListener(this);
        findViewById(R.id.button17).setOnClickListener(this);
        findViewById(R.id.button18).setOnClickListener(this);
        findViewById(R.id.button19).setOnClickListener(this);
    }

    @Override
    protected void onStart() {
        Log.d(TAG, "1026 onStart: " + wbpProtocol);
        super.onStart();
        wbpProtocol.setOnConnectStateListener(this);
        wbpProtocol.setOnDataResponseListener(this);
        wbpProtocol.setOnNotifyStateListener(this);
        wbpProtocol.setOnWriteStateListener(this);

        startScan();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (wbpProtocol.isConnected()) wbpProtocol.disconnect();
        wbpProtocol.stopScan();
    }

    private void startScan() {
        if (!wbpProtocol.isSupportBluetooth(this)) {
            Log.d(TAG, "1026 not support Bluetooth");
            return;
        }
        Log.d(TAG, "1026 start scan");
        logListAdapter.addLog("start scan");
        wbpProtocol.startScan(10);
    }

    @Override
    protected void onStop() {
        super.onStop();
        wbpProtocol.disconnect();
        wbpProtocol.stopScan();
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        onStop();
        //wbpProtocol.stopScan();
    }

    static int loopsKey = 0;
    @Override
    public void onClick(View v) {
        Button btn = (Button)findViewById(v.getId());
        logListAdapter.addLog("WRITE : " + btn.getText().toString());

        if (wbpProtocol.isConnected()) {
            switch (v.getId()) {
                case R.id.button1:
                    wbpProtocol.readUsualModeHistoryData();
                    break;
                case R.id.button2:
                    wbpProtocol.readDiagnosticModeHistoryData();
                    break;
                case R.id.button3://04h
                    Boolean clearUsualMode = selectedBOOL();
                    Boolean clearDiagnosticlMode = selectedBOOL();
                    Boolean clearNocturnalMode = selectedBOOL();
                    logListAdapter.addLog("WRITE : " + "clearHistoryData\nclearUsualMode：" + clearUsualMode +
                            "\nclearDiagnosticlMode：" + clearDiagnosticlMode +
                            "\nclearNocturnalMode：" + clearNocturnalMode);
                    wbpProtocol.clearHistoryData(clearUsualMode, clearDiagnosticlMode, clearNocturnalMode);
                    break;
                case R.id.button4://05h
                    wbpProtocol.clearCurrentModeHistoryData();
                    break;
                case R.id.button5:
                    wbpProtocol.disconnectWBP();
                    break;
                case R.id.button6:
                    wbpProtocol.writeDeviceTime();
                    break;
                case R.id.button7:
                    String userID = produceUserId();
                    logListAdapter.addLog("WRITE : " + "writeUserID：" + userID);
                    wbpProtocol.writeUserID(userID);
                    break;
                case R.id.button8:
                    wbpProtocol.readNocturnalModeSetting();
                    break;
                case R.id.button9:
                    Boolean openNocturnal = selectedBOOL();
                    Calendar today = Calendar.getInstance();
                    int YEAR = today.get(Calendar.YEAR);
                    int MONTH = today.get(Calendar.MONTH);
                    int DATE = today.get(Calendar.DATE);
                    int HOUR = selectedhour();
                    logListAdapter.addLog("WRITE : " + "changeNocturnalModeSetting open Nocturnal：" + openNocturnal +
                            " YEAR：" + YEAR + " MONTH：" + MONTH + " DATE：" + DATE + " HOUR：" + HOUR);
                    wbpProtocol.changeNocturnalModeSetting(openNocturnal, YEAR, MONTH, DATE, HOUR);
                    break;
                case R.id.button11:
                    wbpProtocol.readDeviceIDAndInfo();
                    break;
                case R.id.button12:
                    wbpProtocol.readDeviceTime();
                    break;
                case R.id.button13:
                    wbpProtocol.readUserAndVersionData();
                    break;
                case R.id.button14:
                    wbpProtocol.readNocturnalModeHistoryData();
                    break;
                case R.id.button15:
                    wbpProtocol.readSerialNumber();
                    break;
                case R.id.button16:
                    String sn = produceSN();
                    logListAdapter.addLog("WRITE : writeSerialNumber：" + sn);
//                    Global.wbpProtocol.writeSerialNumber(sn);
                    break;
                case R.id.button17:     // CMD 0x0800
                    if (protocolVersion > 13){
                        logListAdapter.addLog("Read : Read Measurement Setting：");
                        wbpProtocol.readMeasurementSetting();
                    }
                    break;
                case R.id.button18:     // CMD 0x0801
                    if (protocolVersion > 13) {
                        logListAdapter.addLog("WRITE : write Measurement Setting：");
                        getDataProgress();
                    }
                    break;

                case R.id.button19:     // CMD 0x00F1
                    if (protocolVersion >= 15)
                    {
                        logListAdapter.addLog("Read : Read usual mode history data each measurement：");
                        wbpProtocol.readUsualModeHistoryDataEachMeasurement();
                    }
                    break;

            }
        }
    }


    private void getDataProgress()
    {
        npDialog = new NumerPickerDialog();
        npDialog.setDialogValue(3, 0, 0, "Measure Times");
        loopsKey = 0;
        npDialog.show(this);
    }

    public void cancelClick(View v)
    {
        npDialog.dismiss();
    }

    int mTimes=0, reset=0, interval=0, eAverage=0, swAfib=0 ;
    NumerPickerDialog np1, np2, np3, np4;
    public void dialogButtonOnClick(View v) {

        switch (v.getId()) {
            case R.id.button_common_alert_cancel:
                npDialog.dismiss();
                loopsKey = 0;
                break;

            case R.id.button_common_alert_ok:
                Log.w(TAG, "loopsKey: " + loopsKey);

                switch (loopsKey) {
                    case 0:
                        mTimes = npDialog.mCurrent;
                        npDialog.dismiss();
                        np1 = new NumerPickerDialog();
                        np1.setDialogValue(restIntervals.length-1, 0, 0, "Reset Time");
                        np1.show(this);
                        break;


                    case 1:
                        reset = np1.mCurrent;
                        np1.dismiss();
                        np2 = new NumerPickerDialog();
                        np2.setDialogValue(restIntervals.length-1, 0, 1, "Interval Time");
                        np2.show(this);
                        break;

                    case 2:
                        interval = np2.mCurrent;
                        np2.dismiss();

                        np3 = new NumerPickerDialog();
                        np3.setDialogValue(1, 0, 0, "Exclude Average");
                        np3.show(this);
                        break;

                    case 3:
                        eAverage = np3.mCurrent;
                        np3.dismiss();
                        np4 = new NumerPickerDialog();
                        np4.setDialogValue(1, 0, 0, "AFIB ON/OFF");
                        np4.show(this);
                        break;

                    case 4:
                        swAfib = np4.mCurrent;
                        np4.dismiss();
                        Log.w(TAG, "mTimes: " + mTimes + ", reset: " + reset +
                                ", interval: " + interval + ", eAverage: " + eAverage + ", swAfib: " + swAfib);

                        wbpProtocol.writeMeasurementSetting(mTimes, reset, interval, eAverage, swAfib);

                        try {
                            Thread.sleep(1000);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                        wbpProtocol.readMeasurementSetting();
                        break;

                    case 5:
                        loopsKey = 100;
                        break;
                }
                if (loopsKey < 6)
                    loopsKey++;

                break;
            }

    }

    public static String produceUserId() {
        int[] A = new int[11];
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < 11; i++) {
            if (i < 9) {
                A[i] = (int) ((Math.random() * 10) + 48);
            } else {
                A[i] = (int) ((Math.random() * 26) + 65);
            }
            stringBuilder.append((char)A[i]);
        }
        return String.valueOf(stringBuilder);
    }

    public static String produceSN() {
        int[] A = new int[20];
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < 20; i++) {
            A[i] = (int) ((Math.random() * 10) + 48);
            stringBuilder.append((char)A[i]);
        }
        return String.valueOf(stringBuilder);
    }

    public static  Boolean selectedBOOL() {
        Random ran = new Random();
        Boolean selectedBool = ran.nextInt(2) == 1;
        return selectedBool;
    }

    public static  int selectedhour() {
        int[] hours = {0,1,2,3,4,5,6,22,23};
        Random ran = new Random();
        int h = ran.nextInt(hours.length);
        return hours[h];
    }

    @Override
    public void onWriteMessage(boolean isSuccess, String message) {
        logListAdapter.addLog("WRITE : " + message);

    }

    @Override
    public void onNotifyMessage(String message) {
        logListAdapter.addLog("NOTIFY : " + message);
    }



    @Override
    public void onBtStateChanged(boolean isEnable) {
        //BLE will be returned when it is turned enable or disable
        if (isEnable) {
            Toast.makeText(this, "BLE is enable!!", Toast.LENGTH_SHORT).show();
            startScan();
        } else {
            Toast.makeText(this, "BLE is disable!!", Toast.LENGTH_SHORT).show();
        }
    }

    @Override
    public void onScanResult(String mac, String name, int rssi) {

        if (!name.startsWith("n/a")) {
            logListAdapter.addLog("onScanResult：" + name + " mac:" + mac + ", rssi:"+rssi);
        }

        //Blood pressure machine
        if (name.startsWith("WatchBP Home")) {
            if (isConnecting)
                return;
            isConnecting = true;

            //Stop scanning before connecting
            wbpProtocol.stopScan();

            Log.w(TAG, "Normal connect ....");
            //Connection
            wbpProtocol.connect(mac);
            //Global.wbpProtocol.bond(mac);
            logListAdapter.addLog("connect："+mac);

        }

    }

    @Override
    public void onConnectionState(WBPProtocol.ConnectState state) {
        //BLE connection status return, used to judge connection or disconnection
        switch (state) {
            case Connected:
                isConnecting = false;
                findViewById(R.id.buttonView).setVisibility(View.VISIBLE);
                logListAdapter.addLog("Connected");
                break;
            case ConnectTimeout:
                isConnecting = false;
                findViewById(R.id.buttonView).setVisibility(View.GONE);
                logListAdapter.addLog("ConnectTimeout");
                break;
            case Disconnect:
                isConnecting = false;
                findViewById(R.id.buttonView).setVisibility(View.GONE);
                logListAdapter.addLog("Disconnected");
                startScan();
                break;
            case ScanFinish:
                findViewById(R.id.buttonView).setVisibility(View.GONE);
                logListAdapter.addLog("ScanFinish");
                startScan();
                break;
        }
    }

    @Override
    public void onResponseReadUsualModeHistory(DRecord dRecord, boolean isNullData) {
        logListAdapter.addLog("onResponseReadUsualModeHistory：" + dRecord + "\nis Null Data：" + isNullData);

    }

    @Override
    public void onResponseReadUsualModeHistoryEach(List<EachMData> eachMDataList, boolean isNullData) {
        logListAdapter.addLog("onResponseReadUsualModeHistoryEach：" + eachMDataList + "\nis Null Data：" + isNullData);
    }

    @Override
    public void onResponseReadDiagnosticModeHistory(DiagnosticDRecord dRecord, boolean isNullData) {
        logListAdapter.addLog("onResponseReadDiagnosticModeHistory：" + dRecord + "\nis Null Data：" + isNullData);

    }

    @Override
    public void onResponseClearSelectedModeHistory(boolean isSuccess) {
        logListAdapter.addLog("onResponseClearSelectedModeHistory：" + isSuccess);

    }

    @Override
    public void onResponseClearCurrentModeHistory(boolean isSuccess) {
        logListAdapter.addLog("onResponseClearCurrentModeHistory：" + isSuccess);

    }

    @Override
    public void onResponseWriteDeviceTime(boolean isSuccess) {
        logListAdapter.addLog("onResponseWriteDeviceTime：" + isSuccess);

    }

    @Override
    public void onResponseWriteUserID(boolean isSuccess) {
        logListAdapter.addLog("onResponseWriteUserID：" + isSuccess);

    }

    @Override
    public void onResponseReadNocturnalModeSetting(DeviceInfo deviceInfo) {
        logListAdapter.addLog("onResponseReadNocturnalModeSetting：" + deviceInfo);

    }

    @Override
    public void onResponseChangeNocturnalModeSetting(boolean isSuccess) {
        logListAdapter.addLog("onResponseChangeNocturnalModeSetting：" + isSuccess);

    }

    @Override       // 0800h
    public void onResponseReadMeasurementSetting(DeviceInfo deviceInfo) {
        Log.w(TAG, "onResponseReadMeasurementSetting(), deviceInfo: " + deviceInfo);
        logListAdapter.addLog("onResponseReadMeasurementSetting:" + deviceInfo);
    }

    @Override       // 0801h
    public void onResponseWriteMeasurementSetting(boolean isSuccess) {
        logListAdapter.addLog("onResponseWriteMeasurementSetting：" + isSuccess);
    }

    @Override       // 0A00h
    public void onResponseEraseAllOfMeasureAndErrorTimes(String data) {
        logListAdapter.addLog("onResponseEraseAllOfMeasureAndErrorTimes(), data: " + data);
    }

    @Override
    public void onResponseReadDeviceInfo(DeviceInfo deviceInfo) {
        logListAdapter.addLog("onResponseReadDeviceInfo：" + deviceInfo);

    }

    @Override
    public void onResponseReadDeviceTime(DeviceInfo deviceInfo) {
        logListAdapter.addLog("onResponseReadDeviceTime：" + deviceInfo);

    }

    @Override
    public void onResponseReadUserAndVersionData(User user, VersionData versionData) {
        protocolVersion = versionData.getProtocolVersion();
        Log.w(TAG, "onResponseReadUser: " + user + "\nVersionData:" + versionData);
        logListAdapter.addLog("onResponseReadUser: " + user + "\nVersionData:" + versionData);

        if (protocolVersion > 13)
        {
            findViewById(R.id.button17).setEnabled(true);
            findViewById(R.id.button18).setEnabled(true);
        }
        else if(protocolVersion < 14)
        {
            findViewById(R.id.button17).setEnabled(false);
            findViewById(R.id.button18).setEnabled(false);
        }

        if (protocolVersion > 14) {
            findViewById(R.id.button19).setVisibility(View.VISIBLE);
            findViewById(R.id.button19).setEnabled(true);
        }
        else {
            findViewById(R.id.button19).setEnabled(false);
            findViewById(R.id.button19).setVisibility(View.GONE);
        }
    }

    @Override
    public void onResponseReadNocturnalPatternHistory(NocturnalModeDRecord dRecord, boolean isNullData) {
        logListAdapter.addLog("onResponseReadNocturnalPatternHistory：" + dRecord + "\nis Null Data：" + isNullData);

    }

    @Override
    public void onResponseReadReadSerialNumber(DeviceInfo deviceInfo) {
        logListAdapter.addLog("onResponseReadReadSerialNumber：" + deviceInfo);

    }

    @Override
    public void onResponseWriteSerialNumber(boolean isSuccess) {
        logListAdapter.addLog("onResponseReadReadSerialNumber：" + isSuccess);
    }
}
